--
-- Engine Cover
--
-- author  Mythos
-- date  29/03/2017
-- www.ARM-team.gr
-- Copyright (C) ARM Team, All Rights Reserved.

EngineCover = {};

function EngineCover.prerequisitesPresent(specializations)
    return true;
end;

--________________________________________________________________________________________________________________________________________________________
function EngineCover:load(savegame)
	-- Functions
	self.MPCode = SpecializationUtil.callSpecializationsFunction("MPCode");
	self.playerCallback = SpecializationUtil.callSpecializationsFunction("playerCallback");

	-- Booleans & Vars
	self.SMG = false;
	
	-- Animations
	self.engineAnim = getXMLString(self.xmlFile, "vehicle.engineAnim#name");
	
	-- Triggers
	self.SMGTriggers = {};
	local i=0;
	while true do
		local key = string.format("vehicle.SMGTriggers.SMGTrigger(%d)", i);
		local trigger = Utils.indexToObject(self.components,getXMLString(self.xmlFile, key.. "#index"));
		if trigger == nil then
			break;
		end;
		self.SMGTriggers[i+1] = {};
		self.SMGTriggers[i+1].trigger = trigger;
		self.SMGTriggers[i+1].enable = false;
		
		addTrigger(trigger, "playerCallback", self);				
		i = i + 1;
	end;
	

end;

--________________________________________________________________________________________________________________________________________________________
function EngineCover:delete()
	for i,j in pairs(self.SMGTriggers) do
		if j.trigger ~= nil then
			removeTrigger(j.trigger);
		end;
	end;
end;

--________________________________________________________________________________________________________________________________________________________
function EngineCover:mouseEvent(posX, posY, isDown, isUp, button)
end;

--________________________________________________________________________________________________________________________________________________________
function EngineCover:keyEvent(unicode, sym, modifier, isDown)
end;

--________________________________________________________________________________________________________________________________________________________
function EngineCover:update(dt)
	
	local enable = false;
	local id = 0;
	for i,j in pairs(self.SMGTriggers) do
		if j.enable then
			id = i;
			enable = true;
			break;
		end;
	end;
	
	if enable then
		if id==1 then
			g_currentMission:addHelpButtonText(string.format(g_i18n:getText("ENGINECOVER"), self.typeDesc), InputBinding.ACTIVATE_OBJECT);
			if InputBinding.hasEvent(InputBinding.ACTIVATE_OBJECT) then
				self:MPCode(1, not self.SMG);
			end;
		end;
	end;
	
end;

--________________________________________________________________________________________________________________________________________________________
function EngineCover:updateTick(dt)	
end;

--________________________________________________________________________________________________________________________________________________________
function EngineCover:onDetach()
end;

--________________________________________________________________________________________________________________________________________________________
function EngineCover:MPCode(mode, state, noEventSend)
	EngineCoverEvent.sendEvent(self, mode, state, noEventSend);
	--______________________________________________________________________________
	if mode == 1 then
	--_________________________________ MUDGUARDS MODE _____________________________
		if state then
			self:playAnimation(self.engineAnim, 1, nil, true);
			self.SMG = true;
		else
			self:playAnimation(self.engineAnim, -1, nil, true);
			self.SMG = false;
		end;
	end;

end;

--________________________________________________________________________________________________________________________________________________________
function EngineCover:draw()	
end;

--________________________________________________________________________________________________________________________________________________________
function EngineCover:postLoad(savegame)
	if savegame ~= nil and not savegame.resetVehicles then
		self.SMG = Utils.getNoNil(getXMLBool(savegame.xmlFile, savegame.key.."#SMG"),false);
		
		if self.SMG then
			self:MPCode(1, self.SMG);
		end;
		
	end;
	
	return BaseMission.VEHICLE_LOAD_OK;
end;

--________________________________________________________________________________________________________________________________________________________
function EngineCover:getSaveAttributesAndNodes(nodeIdent)
	local attributes = 'SMG="'..tostring(self.SMG)..'"';

	local nodes = nil;
	
	return attributes,nodes;
end;

--________________________________________________________________________________________________________________________________________________________
function EngineCover:readStream(streamId, connection)

	self.SMG = streamReadBool(streamId);
	self:MPCode(1, self.SMG, true);
	
end;

--________________________________________________________________________________________________________________________________________________________
function EngineCover:writeStream(streamId, connection)
	streamWriteBool(streamId, self.SMG);
end;

--________________________________________________________________________________________________________________________________________________________
function EngineCover:playerCallback(triggerId, otherId, onEnter, onLeave, onStay)
	
	local id = 0;
	for i,j in pairs(self.SMGTriggers) do
		if j.trigger == triggerId then
			id = i;
			break;
		end;
	end;
	
	if id ~= 0 then
		if onEnter and g_currentMission.controlPlayer and g_currentMission.player ~= nil and otherId == g_currentMission.player.rootNode then
			self.SMGTriggers[id].enable = true;
		elseif onLeave then
			self.SMGTriggers[id].enable = false;
		end;
	end
	
end;

--**********************************************************************************************************************************************************************************
EngineCoverEvent = {};
EngineCoverEvent_mt = Class(EngineCoverEvent, Event);

InitEventClass(EngineCoverEvent, "EngineCoverEvent");

function EngineCoverEvent:emptyNew()
    local self = Event:new(EngineCoverEvent_mt);
    self.className="EngineCoverEvent";
    return self;
end;

function EngineCoverEvent:new(vehicle, mode, state)
    local self = EngineCoverEvent:emptyNew()
    self.vehicle = vehicle;
	self.mode = mode;
	self.state = state;
    return self;
end;

function EngineCoverEvent:readStream(streamId, connection)
    local id = streamReadInt32(streamId);
	self.mode = streamReadInt32(streamId);
	self.state = streamReadBool(streamId);
    self.vehicle = networkGetObject(id);
    self:run(connection);
end;

function EngineCoverEvent:writeStream(streamId, connection)
    streamWriteInt32(streamId, networkGetObjectId(self.vehicle));
	streamWriteInt32(streamId, self.mode);
	streamWriteBool(streamId, self.state);
end;

function EngineCoverEvent:run(connection)
	self.vehicle:MPCode(self.mode, self.state, true);
    if not connection:getIsServer() then
        g_server:broadcastEvent(EngineCoverEvent:new(self.vehicle, self.mode, self.state), nil, connection, self.vehicle);
    end;
end;

function EngineCoverEvent.sendEvent(vehicle, mode, state, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(EngineCoverEvent:new(vehicle, mode, state), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(EngineCoverEvent:new(vehicle, mode, state));
		end;
	end;
end;